package contractmanager;

import java.io.*;
import java.util.Calendar;
import java.text.SimpleDateFormat;

public class Contract {
    //local variables (contract data) for use in the class
    private String clientName, clientReference;
    private int clientPackage, clientDataBundle, clientPeriod;
    private boolean clientInternationalCalls;
    public int[][] plans;

    //constructor for init values
    public Contract()
    {
        //init variables to null/empty/default values
        clientName = "";
        clientPackage = -1;
        clientDataBundle = -1;
        clientReference = "";
        clientPeriod = -1;
        clientInternationalCalls = false;
        //init two dimentional array to store the contract costs
        plans = new int[][]{
         {500, 700, 900, -1},
         {650, 850, 1050, -1},
         {850, 1050, 1250, 2000}
        };
        
    }
    
    //public getters and setters for vars
    //returns client name
    public String getClientName()
    {
        return clientName;
    }
    
    //sets client's name
    public boolean setClientName(String n)
    {
        //data validation (not null, not empty, most contain a space, muse be longer than 3 and less than 25), returns false
        if ((n == null) || (n.equals("")) || !n.contains(" ") || n.length() < 3 || n.length() > 25)
        {
            return false;
        }
        else
        {
            clientName = n;
            return true;
        }
        
    }
    
    //returns client's package
    public int getClientPackage()
    {
        return clientPackage;
    }
    
    //sets client's package
    public boolean setClientPackage(String nS)
    {
        //checks to make sure input can be converted to an integer, if not returns false (not a valid input)
        int n;
        if (isStringInt(nS))
        {
            n = Integer.parseInt(nS);
        }
        else
        {
            return false;
        }
        //data validation (if it's not 1, 2 or 3, return false)
        if (n != 1 && n != 2 && n != 3)
        {
            return false;
        }
        else
        {
            clientPackage = n;
            return true;
        }
        
    }
    
    //returns client's data bundle
    public int getClientDataBundle()
    {
        return clientDataBundle;
    }
    
    //sets cleint's data bundle
    public boolean setClientDataBundle(String nS)
    {
        //checks to make sure input can be converted to an integer, if not returns false (not a valid input)
        int n;
        if (isStringInt(nS))
        {
            n = Integer.parseInt(nS);
        }
        else
        {
            return false;
        }
        //data validation (follows data validation rules set by client for the data bundle)
        if (n != 1 && n != 2 && n != 3 && (n != 4 && (n == 1 || n == 2)))
        {
            return false;
        }
        else
        {
            clientDataBundle = n;
            return true;
        }
    }
    
    //returns the client's reference code
    public String getClientReference()
    {
        return clientReference;
    }
    
    //sets the client's reference code
    public boolean setClientReference(String n)
    {
        //data validation to make sure the format is 'AA111A' (Letter, Letter, Number, Number, Number, Letter)
        if (n.length() == 6)
        {
            //data validation
            //OLD VALIDATION USING CHAR VALUES; CHANGED TO CHARACTER CLASS CHECKS
            //if ((n.charAt(0) >= 'A' && n.charAt(0) <= 'Z') && (n.charAt(1) >= 'A' && n.charAt(1) <= 'Z') && (n.charAt(2) >= '0' && n.charAt(2) <= '9') && (n.charAt(3) >= '0' && n.charAt(3) <= '9') && (n.charAt(4) >= '0' && n.charAt(4) <= '9') && (n.charAt(5) >= 'A' && n.charAt(5) <= 'Z'))
            if (Character.isLetter(n.charAt(0)) && Character.isLetter(n.charAt(1)) && Character.isDigit(n.charAt(2)) && Character.isDigit(n.charAt(3)) && Character.isDigit(n.charAt(4)) && Character.isLetter(n.charAt(5)))
            {
                clientReference = n;
                return true;
            }
            else    
            {
               return false;
            }
        }
        else
        {
            return false;
        }
        
    }
    
    //returns the client's contract period
    public int getClientPeriod()
    {
        return clientPeriod;
    }
    
    //sets the client's contract period
    public boolean setClientPeriod(String nS)
    {
        //checks to make sure input can be converted to an integer, if not returns false (not a valid input)
        int n;
        if (isStringInt(nS))
        {
            n = Integer.parseInt(nS);
        }
        else
        {
            return false;
        }
        //checks to make sure the contract length is valid (1, 12, 18 and 24) else returns false
        if (n == 1 || n == 12 || n == 18 || n == 24 )
        {
            //returns false since business accounts can't be one month long
            if ((n == 1 && clientReference.charAt(5) == 'B'))
            {
                return false;
            }
            clientPeriod = n;
            return true;
        }
        else
        {
            return false;
        }
    }
    
    //returns the client's selection on international calls
    public boolean getClientInternationalCalls()
    {
        return clientInternationalCalls;
    }
    
    //sets the clien't selection on international calls
    public boolean setClientInternationalCalls(String n)
    {
        //data validation, yes or no
        if (n.equalsIgnoreCase("y"))
        {
            clientInternationalCalls = true;
            return true;
        }
        else if (n.equalsIgnoreCase("n"))
        {
            clientInternationalCalls = false;
            return true;
        }
        else
        {
            return false;
        }
    }
    
    
    public int getClientMonthlyRate()
    {
        //returns the cost in the package grid
        return plans[clientPackage-1][clientDataBundle-1];
    }
    
    public int getPromotionAmmount()
    {
        
        boolean businessAccount = false;
        //creates the default multiplyer (1)
        double promoMultiplyer = 1;
        //checks to see if they're a business account
        if (clientReference.charAt(5) == 'B')
        {
            businessAccount = true;
        }
        //if they are they get a 10% discount (also does other calculations to see if they apply for a discount in other areas)
        if (businessAccount)
        {
            promoMultiplyer = promoMultiplyer - 0.1;
        }
        else if (clientPeriod == 18 || clientPeriod == 12)
        {
            promoMultiplyer = promoMultiplyer - 0.05;
        }
        else if (clientPeriod == 24)
        {
            promoMultiplyer = promoMultiplyer - 0.1;
        }
        if (clientInternationalCalls)
        {
            promoMultiplyer = promoMultiplyer + 0.15;
        }
        //returns the complete monthly rate including discounts/or adds
        return (int)(getClientMonthlyRate() * promoMultiplyer);
    }
    
    public int getPromotionPercentage()
    {
        //this gets the percentage of the promotion
        boolean businessAccount = false;
        int promoMultiplyer = 0;
        if (clientReference.charAt(5) == 'B')
        {
            businessAccount = true;
        }
        if (businessAccount)
        {
            promoMultiplyer = promoMultiplyer + 10;
        }
        else if (clientPeriod == 18 || clientPeriod == 12)
        {
            promoMultiplyer = promoMultiplyer + 5;
        }
        else if (clientPeriod == 24)
        {
            promoMultiplyer = promoMultiplyer + 10;
        }

        return promoMultiplyer;
    }
    
    public String getPackageString(int i)
    {
        //returns the text for the value of package
        if (i == 1)
        {
            return "Small (300)";
        }
        if (i == 2)
        {
            return "Medium (600)";
        }
        if (i == 3)
        {
            return "Large (1200)";
        }
        return "";
    }
    
    public String getDataString(int i)
    {
        //returns the text for the value of the data bundle
        if (i == 1)
        {
            return "Low (1GB)";
        }
        if (i == 2)
        {
            return "Medium (4GB)";
        }
        if (i == 3)
        {
            return "High (8GB)";
        }
        if (i == 4)
        {
            return "Unlimited";
        }
        return "";
    }
    
    public String getBusinessStatus(String s)
    {
        //returns a string of the business type (Business or Regular)
        if (s.charAt(5) == 'B')
        {
            return "Business";
        }
        if (s.charAt(5) == 'N')
        {
            return "Regular";
        }
        return "";
    }
    
    public String getIntlCallsYesNo(boolean n)
    {
        //returns the string of the value of international calls
        if (n == true)
        {
            return "Yes";
        }
        return "No";

    }
    
    
    private String convertBoolToYN(Boolean n)
    {
        //converts a boolean to a string of Y or N
        if (n)
        {
            return "Y";
        }
        return "N";
    }
    
    //checks if a string can be cast to an int
    private boolean isStringInt(String nS)
    {
        try 
        {
            Integer.parseInt(nS);
        }
        catch(Exception e)
        {
            return false;
        }
        return true;
    }
    
    //writes the whole contract to a line in a text file
    public boolean writeToFile()
    {
       //declares a new file to be used for the writer
       File contractFile = new File("contracts.txt");
       PrintWriter output = null;

       //checks to make sure the file exists, if it doesn't it creates a new one
       if (!contractFile.exists())
       {
           try 
           {
               contractFile.createNewFile();
           }
           catch (Exception e)
           {
               return false;
           }
       }
       //creates a new FileWriter in the file created earlier, also a printWriter
       try
       {
            FileWriter writeToContract = new FileWriter(contractFile, true);
            output = new PrintWriter(writeToContract);
       }
       catch (Exception e)
       {
               
       }
       
       //get date and create the format for it
       Calendar cal = Calendar.getInstance();
       SimpleDateFormat sdf = new SimpleDateFormat("dd-MMM-yyyy");
       
       //actual writing to file
       output.append(sdf.format(cal.getTime()) + "\t" + this.clientPackage + "\t" + this.clientDataBundle + "\t" + this.clientPeriod + "\t" + this.convertBoolToYN(this.clientInternationalCalls) + "\t" + this.clientReference + "\t" + this.getPromotionAmmount() + "\t" + this.clientName + "\n");
       output.close();
       return true;
    }
    
    //used for writing the contract to the screen
    public String formatStringLine(int lineNumber, String lineData, String lineData2)
    {
        //return data needed for the line referenced
        if (lineNumber == 3)
        {
            String spaceLengthData = "| " + lineData;
            return spaceLengthData + getSpaces(spaceLengthData.length(), 45) + "|";
        }
        if (lineNumber == 4)
        {
            return "|      Ref: " + lineData + "         Date: " + lineData2 + " |";
        }
        if (lineNumber == 5)
        {
            String spaceLengthData = "|  Package: " + lineData;
            String spaceLengthData2 = "Data: " + lineData2;
            return spaceLengthData + getSpaces(lineData.length(), 15) + spaceLengthData2 + getSpaces(lineData2.length(), 12) + "|";
        }
        if (lineNumber == 6)
        {
            String spaceLengthData = "|   Period: " + lineData;
            String spaceLengthData2 = "Type: " + lineData2;
            return spaceLengthData + getSpaces(lineData.length(), 15) + spaceLengthData2 + getSpaces(lineData2.length(), 12) + "|";
        }
        if (lineNumber == 8)
        {
            String localPercent = " ";
            if (this.getPromotionPercentage() > 0)
            {
                localPercent = "%";
            }
            String spaceLengthData = "| Discount: " + lineData + localPercent;
            String spaceLengthData2 = "Intl. Calls: " + lineData2;
            return spaceLengthData + getSpaces(lineData.length() + 1, 8) + spaceLengthData2 + getSpaces(lineData2.length(), 12) + "|";
        }
        if (lineNumber == 10)
        {
            String tempString = (this.getPromotionPercentage() <= 0 ? "" : "Discounted ").concat("Monthly Charge: £" + String.format("%.2f", (float)(getPromotionAmmount() / 100)));
            int localOffset = 0;
            //if the length is odd, an offset is needed for the spaces
            if (tempString.length() % 2 == 1)
            {
                localOffset = 1;
            }
            return "|" + getSpaces(0, ((int)Math.round(44 - tempString.length()) / 2)) + tempString + getSpaces(0, ((int)Math.round(44 - tempString.length()) / 2 + localOffset)) + "|";
        }
        return "";
    }
    
    //used for calculating the needed spaces for each line in the printout
    private String getSpaces(int lineLength, int lineSpaces)
    {
        String tempSpaces = "";
        while (lineLength < lineSpaces)
        {
            tempSpaces = tempSpaces + " ";
            lineLength++;
        }
        return tempSpaces;
    }
    
    
}
